﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Analysis.AlgorithmBehavior.Analyzers;
using HeuristicLab.Common;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace AlgorithmBehaviorUnitTests {
  [TestClass]
  public class DistanceMatrixToPointsTest {
    [TestMethod]
    public void TestDistanceMatrixConversionStatic() {
      int nrOfPoints = 3;
      int dim = 2;
      double[][] orgPoints = new double[nrOfPoints][];
      double[][] orgDm = new double[nrOfPoints][];
      double[][] newDm = new double[nrOfPoints][];
      double[][] newPoints = null;

      AllocArray(orgPoints, dim);
      AllocArray(orgDm, nrOfPoints);
      AllocArray(newDm, nrOfPoints);
      //SamplePoints(orgPoints);
      StaticPoints(orgPoints);
      CalculateDistanceMatrix(orgDm, orgPoints);

      newPoints = DistanceMatrixToPoints.MetricMDS(orgDm);

      CalculateDistanceMatrix(newDm, newPoints);
      Console.WriteLine("orgDm:");
      PrintDM(orgDm);
      Console.WriteLine("newDm:");
      PrintDM(newDm);

      for (int i = 0; i < orgDm.Length; i++) {
        for (int j = 0; j < orgDm.Length; j++) {
          double diff = orgDm[i][j] - newDm[i][j];
          Assert.IsTrue(diff.IsAlmost(0.0));
        }
      }
    }

    private static void PrintDM(double[][] dm) {
      for (int i = 0; i < dm.Length; i++) {
        for (int j = 0; j < dm.Length; j++) {
          Console.Write(dm[i][j] + " ");
        }
        Console.WriteLine();
      }
    }

    private static void SamplePoints(double[][] points) {
      Random rand = new Random();

      for (int i = 0; i < points.Length; i++) {
        for (int j = 0; j < points[i].Length; j++) {
          points[i][j] = rand.NextDouble() * 100;
        }
      }
    }

    private static void StaticPoints(double[][] points) {
      points[0][0] = 2;
      points[0][1] = 1;

      points[1][0] = 5;
      points[1][1] = 5;

      points[2][0] = 3;
      points[2][1] = 7;
    }

    private static void CalculateDistanceMatrix(double[][] dm, double[][] points) {
      for (int i = 0; i < points.Length; i++) {
        for (int j = 0; j < points.Length; j++) {
          dm[i][j] = points[i].EuclideanDistance(points[j]);
        }
      }
    }

    private static void AllocArray(double[][] arr, int size) {
      for (int i = 0; i < arr.Length; i++) {
        arr[i] = new double[size];
      }
    }
  }
}
